#!/usr/bin/python
# -*- coding: utf-8 -*-

# SPDX-License-Identifier: MIT

import sys
import unittest

import dbus
import dbus_test_utils

# from test-service.py
class ServiceError(dbus.DBusException):
    """Exception representing a normal "environmental" error"""
    include_traceback = False
    _dbus_error_name = 'com.example.Networking.ServerError'


class DBusExceptionTestCase(unittest.TestCase):
    """Test the DBusException str/unicode behavior with py2"""

    def test_dbus_exception_normal(self):
        """Test the normal Exception behavior"""
        e = dbus.exceptions.DBusException("baaa")
        msg = e.get_dbus_message()
        self.assertEqual(msg, u"baaa")

    def test_dbus_exception_unicode(self):
        """Test that DBusExceptions that take a py2 unicode work"""
        e = dbus.exceptions.DBusException(u"bäää")
        msg = e.get_dbus_message()
        self.assertEqual(msg, u"bäää")

    def test_dbus_exception_convert_str(self):
        """Test that converting a DBusException to str() works as expected"""
        e = dbus.exceptions.DBusException(u"bxxx")
        self.assertEqual(str(e), "bxxx")

    def test_dbus_exception_convert_str_fail(self):
        """Test that a non-ascii Exception fails to convert to str"""
        if sys.getdefaultencoding() == 'ascii':
            self.assertRaises(UnicodeEncodeError,
                    lambda: str(dbus.exceptions.DBusException(u"bä")))
        else:
            self.skipTest("you're using a weird non-ascii "
                    "sys.getdefaultencoding()")

    def test_dbus_exception_convert_unicode(self):
        """Test that converting a DBusEception to unicode works"""
        e = dbus.exceptions.DBusException(u"bäää")
        self.assertEqual(e.get_dbus_message(), u"bäää")
        self.assertEqual(e.__unicode__(), u"bäää")
        self.assertEqual(unicode(e), u"bäää")

    def test_subclass_exception_unicode(self):
        """Test that DBusExceptions that take a py2 unicode work"""
        e = ServiceError(u"bäää")
        msg = e.get_dbus_message()
        self.assertEqual(msg, u"bäää")
        self.assertEqual(
            unicode(e), u"com.example.Networking.ServerError: bäää")


if __name__ == "__main__":
    dbus_test_utils.main()



"""Support code for implementing D-Bus services via GObjects."""

# Copyright (C) 2007 Collabora Ltd. <http://www.collabora.co.uk/>
#
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use, copy,
# modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import sys
from warnings import warn as _warn
_warn(DeprecationWarning("""\
dbus.gobject_service is deprecated, and is not available under Python 3.

Porting from gobject (PyGObject 2) to gi.repository.GObject (PyGObject 3),
and using dbus.gi_service instead of dbus.gobject_service, is recommended.
"""), DeprecationWarning, stacklevel=2)

if 'gi' in sys.modules:
    # this worked in dbus-python 1.0, so preserve the functionality
    from gi.repository import GObject as gobject
else:
    # this worked in dbus-python < 1.0
    import gobject

import dbus.service

class ExportedGObjectType(gobject.GObject.__class__, dbus.service.InterfaceType):
    """A metaclass which inherits from both GObjectMeta and
    `dbus.service.InterfaceType`. Used as the metaclass for `ExportedGObject`.
    """
    def __init__(cls, name, bases, dct):
        gobject.GObject.__class__.__init__(cls, name, bases, dct)
        dbus.service.InterfaceType.__init__(cls, name, bases, dct)

class ExportedGObject(gobject.GObject, dbus.service.Object):
    """A GObject which is exported on the D-Bus.

    Because GObject and `dbus.service.Object` both have custom metaclasses,
    the naive approach using simple multiple inheritance won't work. This
    class has `ExportedGObjectType` as its metaclass, which is sufficient
    to make it work correctly.
    """
    __metaclass__ = ExportedGObjectType

    def __init__(self, conn=None, object_path=None, **kwargs):
        """Initialize an exported GObject.

        :Parameters:
            `conn` : dbus.connection.Connection
                The D-Bus connection or bus
            `object_path` : str
                The object path at which to register this object.
        :Keywords:
            `bus_name` : dbus.service.BusName
                A bus name to be held on behalf of this object, or None.
            `gobject_properties` : dict
                GObject properties to be set on the constructed object.

                Any unrecognised keyword arguments will also be interpreted
                as GObject properties.
            """
        bus_name = kwargs.pop('bus_name', None)
        gobject_properties = kwargs.pop('gobject_properties', None)

        if gobject_properties is not None:
            kwargs.update(gobject_properties)
        gobject.GObject.__init__(self, **kwargs)
        dbus.service.Object.__init__(self, conn=conn,
                                     object_path=object_path,
                                     bus_name=bus_name)



#!/usr/bin/env python

usage = """Usage:
python unix-fd-service.py <file name> &
python unix-fd-client.py
"""

# Copyright (C) 2004-2006 Red Hat Inc. <http://www.redhat.com/>
# Copyright (C) 2005-2007 Collabora Ltd. <http://www.collabora.co.uk/>
# Copyright (C) 2010 Signove <http://www.signove.com>
#
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use, copy,
# modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from gi.repository import GLib

import dbus
import dbus.service
import dbus.mainloop.glib
import sys
import random

class SomeObject(dbus.service.Object):

    counter = 0

    @dbus.service.method("com.example.SampleInterface",
                         in_signature='', out_signature='h')
    def GetFd(self):
        self.counter = (self.counter + 1) % 3

        if self.counter == 0:
            print "sending UnixFd(filelike)"
            return dbus.types.UnixFd(f)
        elif self.counter == 1:
            print "sending int"
            return f.fileno()
        else:
            print "sending UnixFd(int)"
            return dbus.types.UnixFd(f.fileno())

if len(sys.argv) < 2:
    print usage
    sys.exit(1)

f = file(sys.argv[1], "r")

if __name__ == '__main__':
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    session_bus = dbus.SessionBus()
    name = dbus.service.BusName("com.example.SampleService", session_bus)
    object = SomeObject(session_bus, '/SomeObject')

    mainloop = GLib.MainLoop()
    print "Running fd service."
    print usage
    mainloop.run()



#!/usr/bin/env python
# encoding: utf-8

# Copyright © 2016 Collabora Ltd. <http://www.collabora.co.uk/>
#
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use, copy,
# modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from distutils.dir_util import copy_tree, mkpath
from distutils.file_util import copy_file
from setuptools.dist import Distribution
from setuptools import setup, Extension
import os
import subprocess
import sys

if os.path.exists('.version'):
    version = open('.version').read().strip()
else:
    version = subprocess.check_output(['autoconf', '--trace', 'AC_INIT:$2',
        'configure.ac']).decode('utf-8').strip()

class Build(Distribution().get_command_class('build')):
    """Dummy version of distutils build which runs an Autotools build system
    instead.
    """

    def run(self):
        srcdir = os.getcwd()
        builddir = os.path.join(srcdir, self.build_temp)
        configure = os.path.join(srcdir, 'configure')
        mkpath(builddir)

        if not os.path.exists(configure):
            configure = os.path.join(srcdir, 'autogen.sh')

        subprocess.check_call([
                configure,
                '--disable-maintainer-mode',
                'PYTHON=' + sys.executable,
                # Put the documentation, etc. out of the way: we only want
                # the Python code and extensions
                '--prefix=' + os.path.join(builddir, 'prefix'),
            ],
            cwd=builddir)
        make_args = [
            'pythondir=' + os.path.join(srcdir, self.build_lib),
            'pyexecdir=' + os.path.join(srcdir, self.build_lib),
        ]
        subprocess.check_call(['make', '-C', builddir] + make_args)
        subprocess.check_call(['make', '-C', builddir, 'install'] + make_args)

class BuildExt(Distribution().get_command_class('build_ext')):
    def run(self):
        pass

class BuildPy(Distribution().get_command_class('build_py')):
    def run(self):
        pass

dbus_bindings = Extension('_dbus_bindings',
        sources=['dbus_bindings/module.c'])
dbus_glib_bindings = Extension('_dbus_glib_bindings',
        sources=['dbus_glib_bindings/module.c'])

setup(
    name='dbus-python',
    version=version,
    description='Python bindings for libdbus',
    long_description=open('README').read(),
    maintainer='The D-Bus maintainers',
    maintainer_email='dbus@lists.freedesktop.org',
    download_url='http://dbus.freedesktop.org/releases/dbus-python/',
    url='http://www.freedesktop.org/wiki/Software/DBusBindings/#python',
    packages=['dbus'],
    ext_modules=[dbus_bindings, dbus_glib_bindings],
    license='Expat (MIT/X11)',
    classifiers=[
        'Development Status :: 7 - Inactive',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: C',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Object Brokering',
    ],
    cmdclass={
        'build': Build,
        'build_py': BuildPy,
        'build_ext': BuildExt,
    },
    tests_require=['tap.py'],
)
